@@ -7,6 +7,8 @@ from sys import argv,stdout
 import os
 import os.path
 import shutil
+import re
+import json
 
 DB_DIRECTORY = "/tmp/groonga-databases"
 DEFAULT_DB_NAME = "tutorial.db"
@@ -35,25 +37,42 @@ def reconnect(name):
 
 fout = None
 
-def execmd(cmd, fout):
-  a = '> ' + cmd + "\n"
-  stdout.write(a)
+def execmd(command, fout):
+  formatted_command_line = '> ' + command + "\n"
+  stdout.write(formatted_command_line)
   stdout.flush()
-  groonga_process.stdin.write(cmd + "\n")
+  groonga_process.stdin.write(command + "\n")
   groonga_process.stdin.flush()
   if fout:
-    fout.write(a + "  ")
+    fout.write(formatted_command_line + "  ")
+  is_command = re.match("[a-z]", command)
+  if not is_command:
+    return
+  output_buffer = ""
   while True:
     out = select([groonga_process.stdout], [], [], 0.2)
     if len(out[0]):
-      a = groonga_process.stdout.read(1)
-      if a != None:
-        stdout.write(a)
+      char = groonga_process.stdout.read(1)
+      if char is None:
+        stdout.write(output_buffer)
         if fout:
-          if a == '\n':
-            fout.write(a + "  ")
+          fout.write(output_buffer)
+      else:
+        output_buffer += char
+        if char == '\n':
+          if len(output_buffer) < 80:
+            formatted_output = output_buffer
           else:
-            fout.write(a)
+            parsed_output = json.loads(output_buffer)
+            formatted_output = json.dumps(parsed_output,
+                                          indent=2,
+                                          ensure_ascii=False)
+            formatted_output += "\n"
+            formatted_output = formatted_output.encode("utf-8")
+          stdout.write(formatted_output)
+          if fout:
+            fout.write(re.sub("\n", "\n  ", formatted_output))
+          output_buffer = ""
     else:
       stdout.flush()
       break
