@@ -173,7 +173,7 @@ grn_str2timeval(const char *str, uint32_t str_len, grn_timeval *tv)
   return GRN_SUCCESS;
 }
 
-#ifdef ENABLE_MEMORY_DEBUG
+#ifdef USE_MEMORY_DEBUG
 inline static void
 grn_alloc_info_set_backtrace(char *buffer, size_t size)
 {
@@ -314,13 +314,13 @@ grn_alloc_info_free(grn_ctx *ctx)
   ctx->impl->alloc_info = NULL;
 }
 
-#else /* ENABLE_MEMORY_DEBUG */
+#else /* USE_MEMORY_DEBUG */
 #  define grn_alloc_info_add(address)
 #  define grn_alloc_info_change(old_address, new_address)
 #  define grn_alloc_info_check(address)
 #  define grn_alloc_info_dump(ctx)
 #  define grn_alloc_info_free(ctx)
-#endif /* ENABLE_MEMORY_DEBUG */
+#endif /* USE_MEMORY_DEBUG */
 
 #ifdef USE_FAIL_MALLOC
 int grn_fmalloc_prob = 0;
@@ -391,7 +391,7 @@ grn_ctx_loader_clear(grn_ctx *ctx)
 
 #define IMPL_SIZE ((sizeof(struct _grn_ctx_impl) + (grn_pagesize - 1)) & ~(grn_pagesize - 1))
 
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
 static inline int
 grn_msgpack_buffer_write(void *data, const char *buf, unsigned int len)
 {
@@ -412,7 +412,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
 #ifdef USE_DYNAMIC_MALLOC_CHANGE
   grn_ctx_impl_init_malloc(ctx);
 #endif
-#ifdef ENABLE_MEMORY_DEBUG
+#ifdef USE_MEMORY_DEBUG
   ctx->impl->alloc_info = NULL;
 #endif
   ctx->impl->encoding = ctx->encoding;
@@ -479,7 +479,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
   grn_loader_init(&ctx->impl->loader);
   ctx->impl->plugin_path = NULL;
 
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
   msgpack_packer_init(&ctx->impl->msgpacker, ctx, grn_msgpack_buffer_write);
 #endif
 }
@@ -1080,7 +1080,7 @@ get_content_mime_type(grn_ctx *ctx, const char *p, const char *pe)
         ctx->impl->mime_type = "image/jpeg";
       }
       break;
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     case 'm' :
       if (p + 7 == pe && !memcmp(p, "msgpack", 7)) {
         ctx->impl->output_type = GRN_CONTENT_MSGPACK;
@@ -2352,7 +2352,7 @@ grn_assert(grn_ctx *ctx, int cond, const char* file, int line, const char* func)
 const char *
 grn_get_version(void)
 {
-  return GROONGA_VERSION;
+  return GRN_VERSION;
 }
 
 const char *
