@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/* Copyright(C) 2011-2012 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "groonga_in.h"
 #include <sys/types.h>
@@ -24,16 +24,7 @@
 #include "dat.h"
 #include "util.h"
 
-/*
-  When this code is compiled with MinGW, a macro "open" is defined to replace
-  "open()" by "_open()". This causes a critical problem because grn::dat::Trie
-  and grn::dat::CursorFactory have member functions named "open()". So, the
-  macro must be undefined before the following #includes.
- */
-#ifdef open
-#  undef open
-#endif
-
+#define GRN_DAT_EXPORT
 #include "dat/trie.hpp"
 #include "dat/cursor-factory.hpp"
 
