@@ -79,7 +79,7 @@ grn_output_array_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_typ
     if (DEPTH > 2) { GRN_TEXT_PUTS(ctx, outbuf, "[\t"); }
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     if (nelements < 0) {
       GRN_LOG(ctx, GRN_LOG_DEBUG,
               "grn_output_array_open nelements (%d) for <%s>",
@@ -146,7 +146,7 @@ grn_output_map_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
     if (DEPTH > 2) { GRN_TEXT_PUTS(ctx, outbuf, "{\t"); }
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     if (nelements < 0) {
       GRN_LOG(ctx, GRN_LOG_DEBUG,
               "grn_output_map_open nelements (%d) for <%s>",
@@ -211,7 +211,7 @@ grn_output_int32(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, in
     GRN_TEXT_PUTS(ctx, outbuf, "</INT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_int32(&ctx->impl->msgpacker, value);
 #endif
     break;
@@ -238,7 +238,7 @@ grn_output_int64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, in
     GRN_TEXT_PUTS(ctx, outbuf, "</INT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_int64(&ctx->impl->msgpacker, value);
 #endif
     break;
@@ -265,7 +265,7 @@ grn_output_uint64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, i
     GRN_TEXT_PUTS(ctx, outbuf, "</INT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_uint64(&ctx->impl->msgpacker, value);
 #endif
     break;
@@ -292,7 +292,7 @@ grn_output_float(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, do
     GRN_TEXT_PUTS(ctx, outbuf, "</FLOAT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_double(&ctx->impl->msgpacker, value);
 #endif
     break;
@@ -320,7 +320,7 @@ grn_output_str(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
     GRN_TEXT_PUTS(ctx, outbuf, "</TEXT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_raw(&ctx->impl->msgpacker, value_len);
     msgpack_pack_raw_body(&ctx->impl->msgpacker, value, value_len);
 #endif
@@ -355,7 +355,7 @@ grn_output_bool(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, cha
     GRN_TEXT_PUTS(ctx, outbuf, "</BOOL>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     if (value) {
       msgpack_pack_true(&ctx->impl->msgpacker);
     } else {
@@ -385,7 +385,7 @@ grn_output_void(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
       GRN_TEXT_PUTS(ctx, outbuf, "<NULL/>");
       break;
     case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
       msgpack_pack_nil(&ctx->impl->msgpacker);
 #endif
       break;
@@ -417,7 +417,7 @@ grn_output_time(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, int
     GRN_TEXT_PUTS(ctx, outbuf, "</DATE>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     msgpack_pack_double(&ctx->impl->msgpacker, dv);
 #endif
     break;
@@ -465,7 +465,7 @@ grn_output_geo_point(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type
     GRN_TEXT_PUTS(ctx, outbuf, "</GEO_POINT>");
     break;
   case GRN_CONTENT_MSGPACK :
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     if (value) {
       grn_obj buf;
       GRN_TEXT_INIT(&buf, 0);
@@ -950,7 +950,7 @@ grn_output_obj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
                                                    GRN_CURSOR_ASCENDING);
       int resultset_size = -1;
       if (!tc) { ERRCLR(ctx); }
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
       resultset_size = 1; /* [NHITS, (COLUMNS), (HITS)] */
       if (format->flags & GRN_OBJ_FORMAT_WITH_COLUMN_NAMES) {
         resultset_size++;
