@@ -1188,7 +1188,7 @@ proc_column_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_da
     grn_hash *cols;
     grn_obj *col;
     int column_list_size = -1;
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     column_list_size = 1; /* [header, (key), (COLUMNS)] */
     if ((col = grn_obj_column(ctx, table, KEY_NAME, sizeof(KEY_NAME)-1))) {
       column_list_size++;
@@ -1330,7 +1330,7 @@ proc_table_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
   grn_table_cursor *cur;
   int table_list_length = -1;
 
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
   if (ctx->impl->output_type == GRN_CONTENT_MSGPACK) {
     table_list_length = 1; /* header */
     if ((cur = grn_table_cursor_open(ctx, ctx->impl->db, NULL, 0, NULL, 0, 0, -1, 0))) {
@@ -2310,7 +2310,7 @@ dump_table(grn_ctx *ctx, grn_obj *outbuf, grn_obj *table,
 } while (0)
 
 static void
-dump_scheme(grn_ctx *ctx, grn_obj *outbuf)
+dump_schema(grn_ctx *ctx, grn_obj *outbuf)
 {
   grn_obj *db = ctx->impl->db;
   grn_table_cursor *cur;
@@ -2336,6 +2336,12 @@ dump_scheme(grn_ctx *ctx, grn_obj *outbuf)
           break;
         }
         grn_obj_unlink(ctx, object);
+      } else {
+        /* XXX: this clause is executed when MeCab tokenizer is enabled in
+           database but the groonga isn't supported MeCab.
+           We should return error mesage about it and error exit status
+           but it's too difficult for this architecture. :< */
+        ERRCLR(ctx);
       }
     }
     grn_table_cursor_close(ctx, cur);
@@ -2425,6 +2431,12 @@ dump_all_records(grn_ctx *ctx, grn_obj *outbuf)
       if ((table = grn_ctx_at(ctx, id))) {
         dump_records(ctx, outbuf, table);
         grn_obj_unlink(ctx, table);
+      } else {
+        /* XXX: this clause is executed when MeCab tokenizer is enabled in
+           database but the groonga isn't supported MeCab.
+           We should return error mesage about it and error exit status
+           but it's too difficult for this architecture. :< */
+        ERRCLR(ctx);
       }
     }
     grn_table_cursor_close(ctx, cur);
@@ -2436,8 +2448,8 @@ proc_dump(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
   grn_obj *outbuf = ctx->impl->outbuf;
   ctx->impl->output_type = GRN_CONTENT_NONE;
-  dump_scheme(ctx, outbuf);
-  /* To update index columns correctly, we first create the whole scheme, then
+  dump_schema(ctx, outbuf);
+  /* To update index columns correctly, we first create the whole schema, then
      load non-derivative records, while skipping records of index columns. That
      way, groonga will silently do the job of updating index columns for us. */
   if (GRN_TEXT_LEN(VAR(0)) > 0) {
