@@ -127,11 +127,12 @@ grn_rc
 grn_ra_truncate(grn_ctx *ctx, grn_ra *ra)
 {
   grn_rc rc;
+  const char *io_path;
   char *path;
   unsigned int element_size;
-  if ((path = (char *)grn_io_path(ra->io)) && *path != '\0') {
-    if (!(path = GRN_STRDUP(path))) {
-      ERR(GRN_NO_MEMORY_AVAILABLE, "cannot duplicate path.");
+  if ((io_path = grn_io_path(ra->io)) && *io_path != '\0') {
+    if (!(path = GRN_STRDUP(io_path))) {
+      ERR(GRN_NO_MEMORY_AVAILABLE, "cannot duplicate path: <%s>", io_path);
       return GRN_NO_MEMORY_AVAILABLE;
     }
   } else {
@@ -399,12 +400,13 @@ grn_rc
 grn_ja_truncate(grn_ctx *ctx, grn_ja *ja)
 {
   grn_rc rc;
+  const char *io_path;
   char *path;
   unsigned int max_element_size;
   uint32_t flags;
-  if ((path = (char *)grn_io_path(ja->io)) && *path != '\0') {
-    if (!(path = GRN_STRDUP(path))) {
-      ERR(GRN_NO_MEMORY_AVAILABLE, "cannot duplicate path.");
+  if ((io_path = grn_io_path(ja->io)) && *io_path != '\0') {
+    if (!(path = GRN_STRDUP(io_path))) {
+      ERR(GRN_NO_MEMORY_AVAILABLE, "cannot duplicate path: <%s>", io_path);
       return GRN_NO_MEMORY_AVAILABLE;
     }
   } else {
@@ -1044,7 +1046,7 @@ grn_ja_element_info(grn_ctx *ctx, grn_ja *ja, grn_id id,
   return GRN_SUCCESS;
 }
 
-#ifndef NO_ZLIB
+#ifdef WITH_ZLIB
 #include <zlib.h>
 
 static void *
@@ -1088,9 +1090,9 @@ grn_ja_ref_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *v
   }
   return value;
 }
-#endif /* NO_ZLIB */
+#endif /* WITH_ZLIB */
 
-#ifndef NO_LZO
+#ifdef WITH_LZO
 #include <lzo/lzo1x.h>
 
 static void *
@@ -1126,21 +1128,21 @@ grn_ja_ref_lzo(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *va
   *value_len = lout_len;
   return value;
 }
-#endif /* NO_LZO */
+#endif /* WITH_LZO */
 
 void *
 grn_ja_ref(grn_ctx *ctx, grn_ja *ja, grn_id id, grn_io_win *iw, uint32_t *value_len)
 {
-#ifndef NO_ZLIB
+#ifdef WITH_ZLIB
   if (ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
     return grn_ja_ref_zlib(ctx, ja, id, iw, value_len);
   }
-#endif /* NO_ZLIB */
-#ifndef NO_LZO
+#endif /* WITH_ZLIB */
+#ifdef WITH_LZO
   if (ja->header->flags & GRN_OBJ_COMPRESS_LZO) {
     return grn_ja_ref_lzo(ctx, ja, id, iw, value_len);
   }
-#endif /* NO_LZO */
+#endif /* WITH_LZO */
   return grn_ja_ref_raw(ctx, ja, id, iw, value_len);
 }
 
@@ -1174,7 +1176,7 @@ exit :
   return value;
 }
 
-#ifndef NO_ZLIB
+#ifdef WITH_ZLIB
 inline static grn_rc
 grn_ja_put_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id,
                 void *value, uint32_t value_len, int flags, uint64_t *cas)
@@ -1220,9 +1222,9 @@ grn_ja_put_zlib(grn_ctx *ctx, grn_ja *ja, grn_id id,
   GRN_FREE(zvalue);
   return rc;
 }
-#endif /* NO_ZLIB */
+#endif /* WITH_ZLIB */
 
-#ifndef NO_LZO
+#ifdef WITH_LZO
 inline static grn_rc
 grn_ja_put_lzo(grn_ctx *ctx, grn_ja *ja, grn_id id,
                void *value, uint32_t value_len, int flags, uint64_t *cas)
@@ -1249,22 +1251,22 @@ grn_ja_put_lzo(grn_ctx *ctx, grn_ja *ja, grn_id id,
   GRN_FREE(lvalue);
   return rc;
 }
-#endif /* NO_LZO */
+#endif /* WITH_LZO */
 
 grn_rc
 grn_ja_put(grn_ctx *ctx, grn_ja *ja, grn_id id, void *value, uint32_t value_len,
            int flags, uint64_t *cas)
 {
-#ifndef NO_ZLIB
+#ifdef WITH_ZLIB
   if (ja->header->flags & GRN_OBJ_COMPRESS_ZLIB) {
     return grn_ja_put_zlib(ctx, ja, id, value, value_len, flags, cas);
   }
-#endif /* NO_ZLIB */
-#ifndef NO_LZO
+#endif /* WITH_ZLIB */
+#ifdef WITH_LZO
   if (ja->header->flags & GRN_OBJ_COMPRESS_LZO) {
     return grn_ja_put_lzo(ctx, ja, id, value, value_len, flags, cas);
   }
-#endif /* NO_LZO */
+#endif /* WITH_LZO */
   return grn_ja_put_raw(ctx, ja, id, value, value_len, flags, cas);
 }
 
