@@ -1,4 +1,4 @@
-/* -*- c-basic-offset: 2 -*- */
+/* -*- c-basic-offset: 2; indent-tabs-mode: nil -*- */
 /* Copyright(C) 2010-2012 Brazil
 
   This library is free software; you can redistribute it and/or
@@ -12,7 +12,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "ctx.h"
@@ -27,16 +27,57 @@
 #define CONST_STR_LEN(x) x, x ? sizeof(x) - 1 : 0
 #define TEXT_VALUE_LEN(x) GRN_TEXT_VALUE(x), GRN_TEXT_LEN(x)
 
+#define MIN_LEARN_DISTANCE (60 * GRN_TIME_USEC_PER_SEC)
+
 #define COMPLETE 1
 #define CORRECT  2
 #define SUGGEST  4
 
 typedef enum {
-  GRN_SUGGEST_PREFIX_SEARCH_YES,
-  GRN_SUGGEST_PREFIX_SEARCH_NO,
-  GRN_SUGGEST_PREFIX_SEARCH_AUTO
-} grn_suggest_prefix_search_mode;
+  GRN_SUGGEST_SEARCH_YES,
+  GRN_SUGGEST_SEARCH_NO,
+  GRN_SUGGEST_SEARCH_AUTO
+} grn_suggest_search_mode;
 
+typedef struct {
+  grn_obj *post_event;
+  grn_obj *post_type;
+  grn_obj *post_item;
+  grn_obj *seq;
+  grn_obj *post_time;
+  grn_obj *pairs;
+
+  int learn_distance_in_seconds;
+
+  grn_id post_event_id;
+  grn_id post_type_id;
+  grn_id post_item_id;
+  grn_id seq_id;
+  int64_t post_time_value;
+
+  grn_obj *seqs;
+  grn_obj *seqs_events;
+  grn_obj *events;
+  grn_obj *events_item;
+  grn_obj *events_type;
+  grn_obj *events_time;
+  grn_obj *event_types;
+  grn_obj *items;
+  grn_obj *items_freq;
+  grn_obj *items_freq2;
+  grn_obj *items_last;
+  grn_obj *pairs_pre;
+  grn_obj *pairs_post;
+  grn_obj *pairs_freq0;
+  grn_obj *pairs_freq1;
+  grn_obj *pairs_freq2;
+
+  grn_obj v1;
+  grn_obj pre_events;
+
+  uint64_t key_prefix;
+  grn_obj pre_item;
+} grn_suggest_learner;
 
 static int
 grn_parse_suggest_types(const char *nptr, const char *end)
@@ -228,7 +269,7 @@ complete(grn_ctx *ctx, grn_obj *items, grn_obj *items_boost, grn_obj *col,
          grn_obj *query, grn_obj *sortby,
          grn_obj *output_columns, int offset, int limit,
          int frequency_threshold, double conditional_probability_threshold,
-         grn_suggest_prefix_search_mode prefix_search_mode)
+         grn_suggest_search_mode prefix_search_mode)
 {
   grn_obj *res;
   grn_obj *items_freq = grn_obj_column(ctx, items, CONST_STR_LEN("freq"));
@@ -273,8 +314,8 @@ complete(grn_ctx *ctx, grn_obj *items, grn_obj *items_boost, grn_obj *col,
       cooccurrence_search(ctx, items, items_boost, tid, res, COMPLETE,
                           frequency_threshold,
                           conditional_probability_threshold);
-      if (((prefix_search_mode == GRN_SUGGEST_PREFIX_SEARCH_YES) ||
-           (prefix_search_mode == GRN_SUGGEST_PREFIX_SEARCH_AUTO &&
+      if (((prefix_search_mode == GRN_SUGGEST_SEARCH_YES) ||
+           (prefix_search_mode == GRN_SUGGEST_SEARCH_AUTO &&
             !grn_table_size(ctx, res))) &&
           (cur = grn_table_cursor_open(ctx, items, norm->norm, norm->norm_blen,
                                        NULL, 0, 0, -1, GRN_CURSOR_PREFIX))) {
@@ -300,7 +341,8 @@ static void
 correct(grn_ctx *ctx, grn_obj *items, grn_obj *items_boost,
         grn_obj *query, grn_obj *sortby,
         grn_obj *output_columns, int offset, int limit,
-        int frequency_threshold, double conditional_probability_threshold)
+        int frequency_threshold, double conditional_probability_threshold,
+        grn_suggest_search_mode similar_search_mode)
 {
   grn_obj *res;
   grn_obj *items_freq2 = grn_obj_column(ctx, items, CONST_STR_LEN("freq2"));
@@ -315,7 +357,10 @@ correct(grn_ctx *ctx, grn_obj *items, grn_obj *items_boost,
                                     frequency_threshold,
                                     conditional_probability_threshold);
     LAP(":", "cooccur(%d)", max_score);
-    if (GRN_TEXT_LEN(query) && max_score < frequency_threshold) {
+    if (GRN_TEXT_LEN(query) &&
+        ((similar_search_mode == GRN_SUGGEST_SEARCH_YES) ||
+         (similar_search_mode == GRN_SUGGEST_SEARCH_AUTO &&
+          max_score < frequency_threshold))) {
       grn_obj *key, *index;
       if ((key = grn_obj_column(ctx, items, CONST_STR_LEN("_key")))) {
         if (grn_column_index(ctx, key, GRN_OP_MATCH, &index, 1, NULL)) {
@@ -437,6 +482,26 @@ suggest(grn_ctx *ctx, grn_obj *items, grn_obj *items_boost,
   }
 }
 
+static grn_suggest_search_mode
+parse_search_mode(grn_ctx *ctx, grn_obj *mode_text)
+{
+  grn_suggest_search_mode mode;
+  int mode_length;
+
+  mode_length = GRN_TEXT_LEN(mode_text);
+  if (mode_length == 3 &&
+      strncasecmp("yes", GRN_TEXT_VALUE(mode_text), 3) == 0) {
+    mode = GRN_SUGGEST_SEARCH_YES;
+  } else if (mode_length == 2 &&
+             strncasecmp("no", GRN_TEXT_VALUE(mode_text), 2) == 0) {
+    mode = GRN_SUGGEST_SEARCH_NO;
+  } else {
+    mode = GRN_SUGGEST_SEARCH_AUTO;
+  }
+
+  return mode;
+}
+
 static grn_obj *
 command_suggest(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
@@ -445,10 +510,10 @@ command_suggest(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
   int offset = 0;
   int limit = DEFAULT_LIMIT;
   int frequency_threshold = DEFAULT_FREQUENCY_THRESHOLD;
-  int prefix_search_len;
   double conditional_probability_threshold =
     DEFAULT_CONDITIONAL_PROBABILITY_THRESHOLD;
-  grn_suggest_prefix_search_mode prefix_search_mode;
+  grn_suggest_search_mode prefix_search_mode;
+  grn_suggest_search_mode similar_search_mode;
 
   types = grn_parse_suggest_types(GRN_TEXT_VALUE(VAR(0)), GRN_BULK_CURR(VAR(0)));
   if (GRN_TEXT_LEN(VAR(6)) > 0) {
@@ -465,16 +530,8 @@ command_suggest(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
     conditional_probability_threshold = strtod(GRN_TEXT_VALUE(VAR(9)), NULL);
   }
 
-  prefix_search_len = GRN_TEXT_LEN(VAR(10));
-  if (prefix_search_len == 3 &&
-      strncasecmp("yes", GRN_TEXT_VALUE(VAR(10)), 3) == 0) {
-    prefix_search_mode = GRN_SUGGEST_PREFIX_SEARCH_YES;
-  } else if (prefix_search_len == 2 &&
-             strncasecmp("no", GRN_TEXT_VALUE(VAR(10)), 2) == 0) {
-    prefix_search_mode = GRN_SUGGEST_PREFIX_SEARCH_NO;
-  } else {
-    prefix_search_mode = GRN_SUGGEST_PREFIX_SEARCH_AUTO;
-  }
+  prefix_search_mode = parse_search_mode(ctx, VAR(10));
+  similar_search_mode = parse_search_mode(ctx, VAR(11));
 
   if ((items = grn_ctx_get(ctx, TEXT_VALUE_LEN(VAR(1))))) {
     if ((items_boost = grn_obj_column(ctx, items, CONST_STR_LEN("boost")))) {
@@ -494,7 +551,8 @@ command_suggest(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
         GRN_OUTPUT_CSTR("correct");
         correct(ctx, items, items_boost, VAR(3), VAR(4),
                 VAR(5), offset, limit,
-                frequency_threshold, conditional_probability_threshold);
+                frequency_threshold, conditional_probability_threshold,
+                similar_search_mode);
       }
       if (types & SUGGEST) {
         GRN_OUTPUT_CSTR("suggest");
@@ -515,107 +573,304 @@ command_suggest(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
   return NULL;
 }
 
+static void
+learner_init_values(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  learner->post_event_id = GRN_RECORD_VALUE(learner->post_event);
+  learner->post_type_id = GRN_RECORD_VALUE(learner->post_type);
+  learner->post_item_id = GRN_RECORD_VALUE(learner->post_item);
+  learner->seq_id = GRN_RECORD_VALUE(learner->seq);
+  learner->post_time_value = GRN_TIME_VALUE(learner->post_time);
+}
+
+static void
+learner_init(grn_ctx *ctx, grn_suggest_learner *learner,
+             grn_obj *post_event, grn_obj *post_type, grn_obj *post_item,
+             grn_obj *seq, grn_obj *post_time, grn_obj *pairs)
+{
+  learner->post_event = post_event;
+  learner->post_type = post_type;
+  learner->post_item = post_item;
+  learner->seq = seq;
+  learner->post_time = post_time;
+  learner->pairs = pairs;
+
+  learner->learn_distance_in_seconds = 0;
+
+  learner_init_values(ctx, learner);
+}
+
+static void
+learner_init_columns(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_id events_id, event_types_id;
+  grn_obj *seqs, *events, *post_item, *items, *pairs;
+
+  learner->seqs = seqs = grn_ctx_at(ctx, GRN_OBJ_GET_DOMAIN(learner->seq));
+  learner->seqs_events = grn_obj_column(ctx, seqs, CONST_STR_LEN("events"));
+
+  events_id = grn_obj_get_range(ctx, learner->seqs_events);
+  learner->events = events = grn_ctx_at(ctx, events_id);
+  learner->events_item = grn_obj_column(ctx, events, CONST_STR_LEN("item"));
+  learner->events_type = grn_obj_column(ctx, events, CONST_STR_LEN("type"));
+  learner->events_time = grn_obj_column(ctx, events, CONST_STR_LEN("time"));
+
+  event_types_id = grn_obj_get_range(ctx, learner->events_type);
+  learner->event_types = grn_obj_column(ctx, events, CONST_STR_LEN("time"));
+
+  post_item = learner->post_item;
+  learner->items = items = grn_ctx_at(ctx, GRN_OBJ_GET_DOMAIN(post_item));
+  learner->items_freq = grn_obj_column(ctx, items, CONST_STR_LEN("freq"));
+  learner->items_freq2 = grn_obj_column(ctx, items, CONST_STR_LEN("freq2"));
+  learner->items_last = grn_obj_column(ctx, items, CONST_STR_LEN("last"));
+
+  pairs = learner->pairs;
+  learner->pairs_pre = grn_obj_column(ctx, pairs, CONST_STR_LEN("pre"));
+  learner->pairs_post = grn_obj_column(ctx, pairs, CONST_STR_LEN("post"));
+  learner->pairs_freq0 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq0"));
+  learner->pairs_freq1 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq1"));
+  learner->pairs_freq2 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq2"));
+}
+
+static void
+learner_fin_columns(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_obj_unlink(ctx, learner->seqs);
+  grn_obj_unlink(ctx, learner->seqs_events);
+
+  grn_obj_unlink(ctx, learner->events);
+  grn_obj_unlink(ctx, learner->events_item);
+  grn_obj_unlink(ctx, learner->events_type);
+  grn_obj_unlink(ctx, learner->events_time);
+
+  grn_obj_unlink(ctx, learner->event_types);
+
+  grn_obj_unlink(ctx, learner->items);
+  grn_obj_unlink(ctx, learner->items_freq);
+  grn_obj_unlink(ctx, learner->items_freq2);
+  grn_obj_unlink(ctx, learner->items_last);
+
+  grn_obj_unlink(ctx, learner->pairs_pre);
+  grn_obj_unlink(ctx, learner->pairs_post);
+  grn_obj_unlink(ctx, learner->pairs_freq0);
+  grn_obj_unlink(ctx, learner->pairs_freq1);
+  grn_obj_unlink(ctx, learner->pairs_freq2);
+}
+
+static void
+learner_init_buffers(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  GRN_UINT32_INIT(&(learner->v1), 0);
+  GRN_UINT32_SET(ctx, &(learner->v1), 1);
+
+  GRN_RECORD_INIT(&(learner->pre_events), 0, grn_obj_id(ctx, learner->events));
+}
+
+static void
+learner_fin_buffers(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_obj_unlink(ctx, &(learner->v1));
+  grn_obj_unlink(ctx, &(learner->pre_events));
+}
+
+static void
+learner_init_submit_learn(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_id items_id;
+
+  learner->key_prefix = ((uint64_t)learner->post_item_id) << 32;
+
+  items_id = grn_obj_get_range(ctx, learner->events_item);
+  GRN_RECORD_INIT(&(learner->pre_item), 0, items_id);
+
+  grn_obj_get_value(ctx, learner->seqs_events, learner->seq_id,
+                    &(learner->pre_events));
+}
+
+static void
+learner_fin_submit_learn(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_obj_unlink(ctx, &(learner->pre_item));
+  GRN_BULK_REWIND(&(learner->pre_events));
+}
+
+static grn_bool
+learner_is_valid_input(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  return learner->post_event_id && learner->post_item_id && learner->seq_id;
+}
+
+static void
+learner_increment(grn_ctx *ctx, grn_suggest_learner *learner,
+                  grn_obj *column, grn_id record_id)
+{
+  grn_obj_set_value(ctx, column, record_id, &(learner->v1), GRN_OBJ_INCR);
+}
+
+static void
+learner_increment_item_freq(grn_ctx *ctx, grn_suggest_learner *learner,
+                            grn_obj *column)
+{
+  learner_increment(ctx, learner, column, learner->post_item_id);
+}
+
+static void
+learner_set_last_post_time(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  grn_obj_set_value(ctx, learner->items_last, learner->post_item_id,
+                    learner->post_time, GRN_OBJ_SET);
+}
+
+static void
+learner_learn_for_complete_and_correcnt(grn_ctx *ctx,
+                                        grn_suggest_learner *learner)
+{
+  grn_obj *pre_item, *post_item, *pre_events;
+  grn_obj pre_type, pre_time;
+  grn_id *ep, *es;
+  uint64_t key;
+  int64_t post_time_value;
+
+  pre_item = &(learner->pre_item);
+  post_item = learner->post_item;
+  pre_events = &(learner->pre_events);
+  post_time_value = learner->post_time_value;
+  GRN_RECORD_INIT(&pre_type, 0, grn_obj_get_range(ctx, learner->events_type));
+  GRN_TIME_INIT(&pre_time, 0);
+  ep = (grn_id *)GRN_BULK_CURR(pre_events);
+  es = (grn_id *)GRN_BULK_HEAD(pre_events);
+  while (es < ep--) {
+    grn_id pair_id;
+    int added;
+    int64_t learn_distance;
+
+    GRN_BULK_REWIND(&pre_type);
+    GRN_BULK_REWIND(&pre_time);
+    GRN_BULK_REWIND(pre_item);
+    grn_obj_get_value(ctx, learner->events_type, *ep, &pre_type);
+    grn_obj_get_value(ctx, learner->events_time, *ep, &pre_time);
+    grn_obj_get_value(ctx, learner->events_item, *ep, pre_item);
+    learn_distance = post_time_value - GRN_TIME_VALUE(&pre_time);
+    if (learn_distance >= MIN_LEARN_DISTANCE) {
+      learner->learn_distance_in_seconds =
+        (int)(learn_distance / GRN_TIME_USEC_PER_SEC);
+      break;
+    }
+    key = learner->key_prefix + GRN_RECORD_VALUE(pre_item);
+    pair_id = grn_table_add(ctx, learner->pairs, &key, sizeof(uint64_t),
+                            &added);
+    if (added) {
+      grn_obj_set_value(ctx, learner->pairs_pre, pair_id, pre_item,
+                        GRN_OBJ_SET);
+      grn_obj_set_value(ctx, learner->pairs_post, pair_id, post_item,
+                        GRN_OBJ_SET);
+    }
+    if (GRN_RECORD_VALUE(&pre_type)) {
+      learner_increment(ctx, learner, learner->pairs_freq1, pair_id);
+      break;
+    } else {
+      learner_increment(ctx, learner, learner->pairs_freq0, pair_id);
+    }
+  }
+  GRN_OBJ_FIN(ctx, &pre_type);
+  GRN_OBJ_FIN(ctx, &pre_time);
+}
+
+static void
+learner_learn_for_suggest(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  char keybuf[GRN_TABLE_MAX_KEY_SIZE];
+  int keylen = grn_table_get_key(ctx, learner->items, learner->post_item_id,
+                                 keybuf, GRN_TABLE_MAX_KEY_SIZE);
+  grn_token *token = grn_token_open(ctx, learner->items, keybuf, keylen, 1);
+  if (token) {
+    grn_id tid;
+    grn_obj *pre_item = &(learner->pre_item);
+    grn_obj *post_item = learner->post_item;
+    grn_hash *token_ids = NULL;
+    while ((tid = grn_token_next(ctx, token)) && tid != learner->post_item_id) {
+      uint64_t key;
+      int added;
+      grn_id pair_id;
+      key = learner->key_prefix + tid;
+      pair_id = grn_table_add(ctx, learner->pairs, &key, sizeof(uint64_t),
+                              &added);
+      if (added) {
+	GRN_RECORD_SET(ctx, pre_item, tid);
+	grn_obj_set_value(ctx, learner->pairs_pre, pair_id,
+                          pre_item, GRN_OBJ_SET);
+	grn_obj_set_value(ctx, learner->pairs_post, pair_id,
+                          post_item, GRN_OBJ_SET);
+      }
+      if (!token_ids) {
+        token_ids = grn_hash_create(ctx, NULL, sizeof(grn_id), 0,
+                                    GRN_OBJ_TABLE_HASH_KEY|GRN_HASH_TINY);
+      }
+      if (token_ids) {
+        int token_added;
+        grn_hash_add(ctx, token_ids, &tid, sizeof(grn_id), NULL, &token_added);
+        if (token_added) {
+          learner_increment(ctx, learner, learner->pairs_freq2, pair_id);
+        }
+      }
+    }
+    if (token_ids) {
+      grn_hash_close(ctx, token_ids);
+    }
+    grn_token_close(ctx, token);
+  }
+}
+
+static void
+learner_append_post_event(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  GRN_RECORD_SET(ctx, &(learner->pre_events), learner->post_event_id);
+  grn_obj_set_value(ctx, learner->seqs_events, learner->seq_id,
+                    &(learner->pre_events), GRN_OBJ_APPEND);
+}
+
+static void
+learner_learn(grn_ctx *ctx, grn_suggest_learner *learner)
+{
+  if (learner_is_valid_input(ctx, learner)) {
+    learner_init_columns(ctx, learner);
+    learner_init_buffers(ctx, learner);
+    learner_increment_item_freq(ctx, learner, learner->items_freq);
+    learner_set_last_post_time(ctx, learner);
+    if (learner->post_type_id) {
+      learner_init_submit_learn(ctx, learner);
+      learner_increment_item_freq(ctx, learner, learner->items_freq2);
+      learner_learn_for_complete_and_correcnt(ctx, learner);
+      learner_learn_for_suggest(ctx, learner);
+      learner_fin_submit_learn(ctx, learner);
+    }
+    learner_append_post_event(ctx, learner);
+    learner_fin_buffers(ctx, learner);
+    learner_fin_columns(ctx, learner);
+  }
+}
+
 static grn_obj *
 func_suggest_preparer(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  int r = 0;
+  int learn_distance_in_seconds = 0;
   grn_obj *obj;
   if (nargs == 6) {
-    grn_obj v1, pre_events;
-    grn_id post_event = GRN_RECORD_VALUE(args[0]);
-    grn_id post_type = GRN_RECORD_VALUE(args[1]);
-    grn_id post_item = GRN_RECORD_VALUE(args[2]);
-    grn_id seq = GRN_RECORD_VALUE(args[3]);
-    int64_t post_time = GRN_TIME_VALUE(args[4]);
+    grn_obj *post_event = args[0];
+    grn_obj *post_type = args[1];
+    grn_obj *post_item = args[2];
+    grn_obj *seq = args[3];
+    grn_obj *post_time = args[4];
     grn_obj *pairs = args[5];
-    if (post_event && post_item && seq) {
-      grn_obj *items = grn_ctx_at(ctx, GRN_OBJ_GET_DOMAIN(args[2]));
-      grn_obj *items_freq = grn_obj_column(ctx, items, CONST_STR_LEN("freq"));
-      grn_obj *items_freq2 = grn_obj_column(ctx, items, CONST_STR_LEN("freq2"));
-      grn_obj *items_last = grn_obj_column(ctx, items, CONST_STR_LEN("last"));
-      grn_obj *seqs = grn_ctx_at(ctx, GRN_OBJ_GET_DOMAIN(args[3]));
-      grn_obj *seqs_events = grn_obj_column(ctx, seqs, CONST_STR_LEN("events"));
-      grn_obj *events = grn_ctx_at(ctx, grn_obj_get_range(ctx, seqs_events));
-      grn_obj *events_type = grn_obj_column(ctx, events, CONST_STR_LEN("type"));
-      grn_obj *events_time = grn_obj_column(ctx, events, CONST_STR_LEN("time"));
-      grn_obj *events_item = grn_obj_column(ctx, events, CONST_STR_LEN("item"));
-      grn_obj *pairs_pre = grn_obj_column(ctx, pairs, CONST_STR_LEN("pre"));
-      grn_obj *pairs_post = grn_obj_column(ctx, pairs, CONST_STR_LEN("post"));
-      grn_obj *pairs_freq0 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq0"));
-      grn_obj *pairs_freq1 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq1"));
-      grn_obj *pairs_freq2 = grn_obj_column(ctx, pairs, CONST_STR_LEN("freq2"));
-      GRN_UINT32_INIT(&v1, 0);
-      GRN_UINT32_SET(ctx, &v1, 1);
-      GRN_RECORD_INIT(&pre_events, 0, grn_obj_id(ctx, events));
-      grn_obj_set_value(ctx, items_freq, post_item, &v1, GRN_OBJ_INCR);
-      grn_obj_set_value(ctx, items_last, post_item, args[4], GRN_OBJ_SET);
-      if (post_type) {
-        int added;
-        grn_id pid, tid, *ep, *es;
-        grn_obj pre_type, pre_time, pre_item;
-        uint64_t key, key_ = ((uint64_t)post_item) << 32;
-        grn_obj_set_value(ctx, items_freq2, post_item, &v1, GRN_OBJ_INCR);
-        grn_obj_get_value(ctx, seqs_events, seq, &pre_events);
-        ep = (grn_id *)GRN_BULK_CURR(&pre_events);
-        es = (grn_id *)GRN_BULK_HEAD(&pre_events);
-        GRN_RECORD_INIT(&pre_type, 0, grn_obj_get_range(ctx, events_type));
-        GRN_TIME_INIT(&pre_time, 0);
-        GRN_RECORD_INIT(&pre_item, 0, grn_obj_get_range(ctx, events_item));
-        while (es < ep--) {
-          GRN_BULK_REWIND(&pre_type);
-          GRN_BULK_REWIND(&pre_time);
-          GRN_BULK_REWIND(&pre_item);
-          grn_obj_get_value(ctx, events_type, *ep, &pre_type);
-          grn_obj_get_value(ctx, events_time, *ep, &pre_time);
-          grn_obj_get_value(ctx, events_item, *ep, &pre_item);
-          if (GRN_TIME_VALUE(&pre_time) + 60 * GRN_TIME_USEC_PER_SEC < post_time) {
-            r = (int)((post_time - GRN_TIME_VALUE(&pre_time))/GRN_TIME_USEC_PER_SEC);
-            break;
-          }
-          key = key_ + GRN_RECORD_VALUE(&pre_item);
-          pid = grn_table_add(ctx, pairs, &key, sizeof(uint64_t), &added);
-          if (added) {
-            grn_obj_set_value(ctx, pairs_pre, pid, &pre_item, GRN_OBJ_SET);
-            grn_obj_set_value(ctx, pairs_post, pid, args[2], GRN_OBJ_SET);
-          }
-          if (GRN_RECORD_VALUE(&pre_type)) {
-            grn_obj_set_value(ctx, pairs_freq1, pid, &v1, GRN_OBJ_INCR);
-            break;
-          } else {
-            grn_obj_set_value(ctx, pairs_freq0, pid, &v1, GRN_OBJ_INCR);
-          }
-        }
-        {
-          char keybuf[GRN_TABLE_MAX_KEY_SIZE];
-          int keylen = grn_table_get_key(ctx, items, post_item,
-                                         keybuf, GRN_TABLE_MAX_KEY_SIZE);
-          grn_token *token = grn_token_open(ctx, items, keybuf, keylen, 1);
-          if (token) {
-            while ((tid = grn_token_next(ctx, token)) && tid != post_item) {
-              key = key_ + tid;
-              pid = grn_table_add(ctx, pairs, &key, sizeof(uint64_t), &added);
-              if (added) {
-                GRN_RECORD_SET(ctx, &pre_item, tid);
-                grn_obj_set_value(ctx, pairs_pre, pid, &pre_item, GRN_OBJ_SET);
-                grn_obj_set_value(ctx, pairs_post, pid, args[2], GRN_OBJ_SET);
-              }
-              grn_obj_set_value(ctx, pairs_freq2, pid, &v1, GRN_OBJ_INCR);
-            }
-            grn_token_close(ctx, token);
-          }
-        }
-        GRN_OBJ_FIN(ctx, &pre_type);
-        GRN_OBJ_FIN(ctx, &pre_time);
-        GRN_OBJ_FIN(ctx, &pre_item);
-        GRN_BULK_REWIND(&pre_events);
-      }
-      GRN_RECORD_SET(ctx, &pre_events, post_event);
-      grn_obj_set_value(ctx, seqs_events, seq, &pre_events, GRN_OBJ_APPEND);
-      GRN_OBJ_FIN(ctx, &pre_events);
-      GRN_OBJ_FIN(ctx, &v1);
-    }
+    grn_suggest_learner learner;
+    learner_init(ctx, &learner,
+                 post_event, post_type, post_item, seq, post_time, pairs);
+    learner_learn(ctx, &learner);
+    learn_distance_in_seconds = learner.learn_distance_in_seconds;
+  }
+  if ((obj = GRN_PROC_ALLOC(GRN_DB_UINT32, 0))) {
+    GRN_UINT32_SET(ctx, obj, learn_distance_in_seconds);
   }
-  if ((obj = GRN_PROC_ALLOC(GRN_DB_UINT32, 0))) { GRN_UINT32_SET(ctx, obj, r); }
   return obj;
 }
 
@@ -639,7 +894,8 @@ GRN_PLUGIN_REGISTER(grn_ctx *ctx)
     {CONST_STR_LEN("limit")},
     {CONST_STR_LEN("frequency_threshold")},
     {CONST_STR_LEN("conditional_probability_threshold")},
-    {CONST_STR_LEN("prefix_search")}
+    {CONST_STR_LEN("prefix_search")},
+    {CONST_STR_LEN("similar_search")}
   };
   GRN_TEXT_INIT(&vars[0].value, 0);
   GRN_TEXT_INIT(&vars[1].value, 0);
@@ -652,8 +908,9 @@ GRN_PLUGIN_REGISTER(grn_ctx *ctx)
   GRN_TEXT_INIT(&vars[8].value, 0);
   GRN_TEXT_INIT(&vars[9].value, 0);
   GRN_TEXT_INIT(&vars[10].value, 0);
+  GRN_TEXT_INIT(&vars[11].value, 0);
   grn_proc_create(ctx, CONST_STR_LEN("suggest"), GRN_PROC_COMMAND,
-                  command_suggest, NULL, NULL, 11, vars);
+                  command_suggest, NULL, NULL, 12, vars);
 
   grn_proc_create(ctx, CONST_STR_LEN("suggest_preparer"), GRN_PROC_FUNCTION,
                   func_suggest_preparer, NULL, NULL, 0, NULL);
