@@ -67,9 +67,11 @@ static int batchmode;
 static int number_of_lines = 0;
 static int newdb;
 static int useql;
+static grn_bool is_daemon_mode = GRN_FALSE;
 static int (*do_client)(int argc, char **argv);
 static int (*do_server)(char *path);
-static const char *pidfile_path = NULL;
+static FILE *pid_file = NULL;
+static const char *pid_file_path = NULL;
 static const char *input_path = NULL;
 
 static grn_encoding encoding;
@@ -78,7 +80,17 @@ static int64_t default_match_escalation_threshold;
 static int log_level;
 static uint32_t cache_limit;
 
-#ifdef HAVE_LIBEDIT
+static int
+grn_rc_to_exit_code(grn_rc rc)
+{
+  if (rc == GRN_SUCCESS) {
+    return EXIT_SUCCESS;
+  } else {
+    return EXIT_FAILURE;
+  }
+}
+
+#ifdef WITH_LIBEDIT
 #include <locale.h>
 #include <histedit.h>
 static EditLine   *line_editor = NULL;
@@ -93,7 +105,8 @@ line_editor_prompt(EditLine *e __attribute__((unused)))
 }
 static const wchar_t * const line_editor_editor = L"emacs";
 
-void line_editor_init(int argc __attribute__((unused)), char *argv[])
+static void
+line_editor_init(int argc __attribute__((unused)), char *argv[])
 {
   const char * const HOME_PATH = getenv("HOME");
   const char * const HISTORY_PATH = "/.groonga-history";
@@ -121,7 +134,8 @@ void line_editor_init(int argc __attribute__((unused)), char *argv[])
   el_source(line_editor, NULL);
 }
 
-void line_editor_fin(void)
+static void
+line_editor_fin(void)
 {
   if (line_editor) {
     el_end(line_editor);
@@ -135,7 +149,7 @@ void line_editor_fin(void)
   }
 }
 
-grn_rc
+static grn_rc
 line_editor_fgets(grn_ctx *ctx, grn_obj *buf)
 {
   grn_rc rc = GRN_SUCCESS;
@@ -165,7 +179,7 @@ line_editor_fgets(grn_ctx *ctx, grn_obj *buf)
   }
   return rc;
 }
-#endif /* HAVE_LIBEDIT */
+#endif /* WITH_LIBEDIT */
 
 inline static grn_rc
 prompt(grn_ctx *ctx, grn_obj *buf)
@@ -174,7 +188,7 @@ prompt(grn_ctx *ctx, grn_obj *buf)
   grn_rc rc = GRN_SUCCESS;
   GRN_BULK_REWIND(buf);
   if (!batchmode) {
-#ifdef HAVE_LIBEDIT
+#ifdef WITH_LIBEDIT
     rc = line_editor_fgets(ctx, buf);
 #else
     fprintf(stderr, "> ");
@@ -464,7 +478,7 @@ transform_xml(grn_ctx *ctx, grn_obj *output, grn_obj *transformed)
   GRN_OBJ_FIN(ctx, &columns);
 }
 
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
 typedef struct {
   grn_ctx *ctx;
   grn_obj *buffer;
@@ -593,7 +607,7 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
     }
     break;
   case GRN_CONTENT_MSGPACK:
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
     {
       msgpack_writer_ctx head_writer_ctx;
       msgpack_packer header_packer;
@@ -702,7 +716,7 @@ s_output(grn_ctx *ctx, int flags, void *arg)
 static int
 do_alone(int argc, char **argv)
 {
-  int rc = -1;
+  int exit_code = EXIT_FAILURE;
   char *path = NULL;
   grn_obj *db;
   grn_ctx ctx_, *ctx = &ctx_;
@@ -722,10 +736,12 @@ do_alone(int argc, char **argv)
         grn_ctx_send(ctx, GRN_TEXT_VALUE(&text), GRN_TEXT_LEN(&text), 0);
         if (ctx->stat == GRN_CTX_QUIT) { break; }
       }
-      rc = ctx->rc;
+      exit_code = grn_rc_to_exit_code(ctx->rc);
       grn_obj_unlink(ctx, &text);
     } else {
+      grn_rc rc;
       rc = grn_ctx_sendv(ctx, argc, argv, 0);
+      exit_code = grn_rc_to_exit_code(rc);
     }
     grn_obj_unlink(ctx, &command);
     grn_obj_close(ctx, db);
@@ -733,7 +749,7 @@ do_alone(int argc, char **argv)
     fprintf(stderr, "db open failed (%s): %s\n", path, ctx->errbuf);
   }
   grn_ctx_fin(ctx);
-  return rc;
+  return exit_code;
 }
 
 static int
@@ -773,7 +789,7 @@ c_output(grn_ctx *ctx)
 static int
 g_client(int argc, char **argv)
 {
-  int rc = -1;
+  int exit_code = EXIT_FAILURE;
   grn_ctx ctx_, *ctx = &ctx_;
   const char *hostname = DEFAULT_DEST;
   if (argc > 0 && argv) { hostname = *argv++; argc--; }
@@ -784,14 +800,16 @@ g_client(int argc, char **argv)
       GRN_TEXT_INIT(&text, 0);
       while (prompt(ctx, &text) != GRN_END_OF_DATA) {
         grn_ctx_send(ctx, GRN_TEXT_VALUE(&text), GRN_TEXT_LEN(&text), 0);
-        rc = ctx->rc;
-        if (rc) { break; }
+        exit_code = grn_rc_to_exit_code(ctx->rc);
+        if (ctx->rc != GRN_SUCCESS) { break; }
         if (c_output(ctx)) { goto exit; }
         if (ctx->stat == GRN_CTX_QUIT) { break; }
       }
       grn_obj_unlink(ctx, &text);
     } else {
+      grn_rc rc;
       rc = grn_ctx_sendv(ctx, argc, argv, 0);
+      exit_code = grn_rc_to_exit_code(rc);
       if (c_output(ctx)) { goto exit; }
     }
   } else {
@@ -799,11 +817,161 @@ g_client(int argc, char **argv)
   }
 exit :
   grn_ctx_fin(ctx);
-  return rc;
+  return exit_code;
 }
 
 /* server */
 
+typedef void (*grn_edge_dispatcher_func)(grn_ctx *ctx, grn_edge *edge);
+typedef void (*grn_handler_func)(grn_ctx *ctx, grn_obj *msg);
+
+static grn_com_queue ctx_new;
+static grn_com_queue ctx_old;
+static grn_mutex q_mutex;
+static grn_cond q_cond;
+static uint32_t nthreads = 0, nfthreads = 0, max_nfthreads;
+
+static int
+daemonize(void)
+{
+  int exit_code = EXIT_SUCCESS;
+#ifndef WIN32
+  pid_t pid;
+
+  switch (fork()) {
+  case 0:
+    break;
+  case -1:
+    perror("fork");
+    return EXIT_FAILURE;
+  default:
+    wait(NULL);
+    _exit(EXIT_SUCCESS);
+  }
+  switch (fork()) {
+  case 0:
+    if (pid_file_path) {
+      pid_file = fopen(pid_file_path, "w");
+    }
+    pid = getpid();
+    if (!pid_file) {
+      fprintf(stderr, "%d\n", pid);
+    } else {
+      fprintf(pid_file, "%d\n", pid);
+      fclose(pid_file);
+      pid_file = NULL;
+    }
+    break;
+  case -1:
+    perror("fork");
+    return EXIT_FAILURE;
+  default:
+    _exit(EXIT_SUCCESS);
+  }
+  {
+    int null_fd = GRN_OPEN("/dev/null", O_RDWR, 0);
+    if (null_fd != -1) {
+      dup2(null_fd, 0);
+      dup2(null_fd, 1);
+      dup2(null_fd, 2);
+      if (null_fd > 2) { GRN_CLOSE(null_fd); }
+    }
+  }
+#endif /* WIN32 */
+  return exit_code;
+}
+
+static void
+clean_pid_file(void)
+{
+#ifndef WIN32
+  if (pid_file_path) {
+    unlink(pid_file_path);
+  }
+#endif
+}
+
+static void
+run_server_loop(grn_ctx *ctx, grn_com_event *ev)
+{
+  while (!grn_com_event_poll(ctx, ev, 1000) && grn_gctx.stat != GRN_CTX_QUIT) {
+    grn_edge *edge;
+    while ((edge = (grn_edge *)grn_com_queue_deque(ctx, &ctx_old))) {
+      grn_obj *msg;
+      while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
+        grn_msg_close(&edge->ctx, msg);
+      }
+      while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
+        grn_msg_close(ctx, msg);
+      }
+      grn_ctx_fin(&edge->ctx);
+      if (edge->com->has_sid && edge->com->opaque == edge) {
+        grn_com_close(ctx, edge->com);
+      }
+      grn_edges_delete(ctx, edge);
+    }
+    /* todo : log stat */
+  }
+  for (;;) {
+    MUTEX_LOCK(q_mutex);
+    if (nthreads == nfthreads) { break; }
+    MUTEX_UNLOCK(q_mutex);
+    usleep(1000);
+  }
+  {
+    grn_edge *edge;
+    GRN_HASH_EACH(ctx, grn_edges, id, NULL, NULL, &edge, {
+      grn_obj *obj;
+      while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
+        grn_msg_close(&edge->ctx, obj);
+      }
+      while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
+        grn_msg_close(ctx, obj);
+      }
+      grn_ctx_fin(&edge->ctx);
+      if (edge->com->has_sid) {
+        grn_com_close(ctx, edge->com);
+      }
+      grn_edges_delete(ctx, edge);
+    });
+  }
+  {
+    grn_com *com;
+    GRN_HASH_EACH(ctx, ev->hash, id, NULL, NULL, &com, { grn_com_close(ctx, com); });
+  }
+}
+
+static int
+run_server(grn_ctx *ctx, grn_obj *db, grn_com_event *ev,
+           grn_edge_dispatcher_func dispatcher, grn_handler_func handler)
+{
+  int exit_code = EXIT_SUCCESS;
+  struct hostent *he;
+  if (!(he = gethostbyname(hostname))) {
+    SERR("gethostbyname");
+  } else {
+    ev->opaque = db;
+    grn_edges_init(ctx, dispatcher);
+    if (!grn_com_sopen(ctx, ev, bind_address, port, handler, he)) {
+      if (is_daemon_mode) {
+        exit_code = daemonize();
+      }
+      if (exit_code == EXIT_SUCCESS) {
+        run_server_loop(ctx, ev);
+      }
+      if (is_daemon_mode) {
+        clean_pid_file();
+      }
+      exit_code = EXIT_SUCCESS;
+    } else {
+      fprintf(stderr, "grn_com_sopen failed (%s:%d): %s\n",
+              bind_address, port, ctx->errbuf);
+    }
+    grn_edges_fin(ctx);
+  }
+  return exit_code;
+}
+
 #define JSON_CALLBACK_PARAM "callback"
 
 typedef struct {
@@ -1267,7 +1435,7 @@ do_mbreq(grn_ctx *ctx, grn_edge *edge)
       cache_init(ctx);
       rid = grn_table_get(ctx, cache_table, key, keylen);
       if (!rid) {
-        // GRN_LOG(ctx, GRN_LOG_NOTICE, "GET k=%d not found", keylen);
+        /* GRN_LOG(ctx, GRN_LOG_NOTICE, "GET k=%d not found", keylen); */
         GRN_MSG_MBRES({
           MBRES(ctx, re, MBRES_KEY_ENOENT, 0, 0, 0);
         });
@@ -1517,12 +1685,6 @@ enum {
   EDGE_ABORT = 0x03,
 };
 
-static grn_com_queue ctx_new;
-static grn_com_queue ctx_old;
-static grn_mutex q_mutex;
-static grn_cond q_cond;
-static uint32_t nthreads = 0, nfthreads = 0, max_nfthreads;
-
 static void * CALLBACK
 h_worker(void *arg)
 {
@@ -1587,7 +1749,7 @@ h_handler(grn_ctx *ctx, grn_obj *msg)
 static int
 h_server(char *path)
 {
-  int rc = -1;
+  int exit_code = EXIT_FAILURE;
   grn_com_event ev;
   grn_ctx ctx_, *ctx = &ctx_;
   grn_ctx_init(ctx, 0);
@@ -1601,7 +1763,7 @@ h_server(char *path)
     struct rlimit lim;
     lim.rlim_cur = 4096;
     lim.rlim_max = 4096;
-    // RLIMIT_OFILE
+    /* RLIMIT_OFILE */
     setrlimit(RLIMIT_NOFILE, &lim);
     lim.rlim_cur = 0;
     lim.rlim_max = 0;
@@ -1613,65 +1775,7 @@ h_server(char *path)
     grn_obj *db;
     db = (newdb || !path) ? grn_db_create(ctx, path, NULL) : grn_db_open(ctx, path);
     if (db) {
-      struct hostent *he;
-      if (!(he = gethostbyname(hostname))) {
-        SERR("gethostbyname");
-      } else {
-        ev.opaque = db;
-        grn_edges_init(ctx, NULL);
-        if (!grn_com_sopen(ctx, &ev, bind_address, port, h_handler, he)) {
-          while (!grn_com_event_poll(ctx, &ev, 1000) && grn_gctx.stat != GRN_CTX_QUIT) {
-            grn_edge *edge;
-            while ((edge = (grn_edge *)grn_com_queue_deque(ctx, &ctx_old))) {
-              grn_obj *msg;
-              while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
-                grn_msg_close(&edge->ctx, msg);
-              }
-              while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
-                grn_msg_close(ctx, msg);
-              }
-              grn_ctx_fin(&edge->ctx);
-              if (edge->com->has_sid && edge->com->opaque == edge) {
-                grn_com_close(ctx, edge->com);
-              }
-              grn_edges_delete(ctx, edge);
-            }
-            // todo : log stat
-          }
-          for (;;) {
-            MUTEX_LOCK(q_mutex);
-            if (nthreads == nfthreads) { break; }
-            MUTEX_UNLOCK(q_mutex);
-            usleep(1000);
-          }
-          {
-            grn_edge *edge;
-            GRN_HASH_EACH(ctx, grn_edges, id, NULL, NULL, &edge, {
-                grn_obj *obj;
-              while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
-                grn_msg_close(&edge->ctx, obj);
-              }
-              while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
-                grn_msg_close(ctx, obj);
-              }
-              grn_ctx_fin(&edge->ctx);
-              if (edge->com->has_sid) {
-                grn_com_close(ctx, edge->com);
-              }
-              grn_edges_delete(ctx, edge);
-            });
-          }
-          {
-            grn_com *com;
-            GRN_HASH_EACH(ctx, ev.hash, id, NULL, NULL, &com, { grn_com_close(ctx, com); });
-          }
-          rc = 0;
-        } else {
-          fprintf(stderr, "grn_com_sopen failed (%s:%d): %s\n",
-                  bind_address, port, ctx->errbuf);
-        }
-        grn_edges_fin(ctx);
-      }
+      exit_code = run_server(ctx, db, &ev, NULL, h_handler);
       grn_obj_close(ctx, db);
     } else {
       fprintf(stderr, "db open failed (%s)\n", path);
@@ -1681,7 +1785,7 @@ h_server(char *path)
     fprintf(stderr, "grn_com_event_init failed\n");
   }
   grn_ctx_fin(ctx);
-  return rc;
+  return exit_code;
 }
 
 static void * CALLBACK
@@ -1749,7 +1853,7 @@ exit :
 }
 
 static void
-dispatcher(grn_ctx *ctx, grn_edge *edge)
+g_dispatcher(grn_ctx *ctx, grn_edge *edge)
 {
   MUTEX_LOCK(q_mutex);
   if (edge->stat == EDGE_IDLE) {
@@ -1820,7 +1924,7 @@ g_handler(grn_ctx *ctx, grn_obj *msg)
       grn_msg_close(ctx, msg);
     } else {
       grn_com_queue_enque(ctx, &edge->recv_new, (grn_com_queue_entry *)msg);
-      dispatcher(ctx, edge);
+      g_dispatcher(ctx, edge);
     }
   }
 }
@@ -1828,7 +1932,7 @@ g_handler(grn_ctx *ctx, grn_obj *msg)
 static int
 g_server(char *path)
 {
-  int rc = -1;
+  int exit_code = EXIT_FAILURE;
   grn_com_event ev;
   grn_ctx ctx_, *ctx = &ctx_;
   grn_ctx_init(ctx, 0);
@@ -1842,7 +1946,7 @@ g_server(char *path)
     struct rlimit lim;
     lim.rlim_cur = 4096;
     lim.rlim_max = 4096;
-    // RLIMIT_OFILE
+    /* RLIMIT_OFILE */
     setrlimit(RLIMIT_NOFILE, &lim);
     lim.rlim_cur = 0;
     lim.rlim_max = 0;
@@ -1854,65 +1958,7 @@ g_server(char *path)
     grn_obj *db;
     db = (newdb || !path) ? grn_db_create(ctx, path, NULL) : grn_db_open(ctx, path);
     if (db) {
-      struct hostent *he;
-      if (!(he = gethostbyname(hostname))) {
-        SERR("gethostbyname");
-      } else {
-        ev.opaque = db;
-        grn_edges_init(ctx, dispatcher);
-        if (!grn_com_sopen(ctx, &ev, bind_address, port, g_handler, he)) {
-          while (!grn_com_event_poll(ctx, &ev, 1000) && grn_gctx.stat != GRN_CTX_QUIT) {
-            grn_edge *edge;
-            while ((edge = (grn_edge *)grn_com_queue_deque(ctx, &ctx_old))) {
-              grn_obj *msg;
-              while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
-                grn_msg_close(&edge->ctx, msg);
-              }
-              while ((msg = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
-                grn_msg_close(ctx, msg);
-              }
-              grn_ctx_fin(&edge->ctx);
-              if (edge->com->has_sid && edge->com->opaque == edge) {
-                grn_com_close(ctx, edge->com);
-              }
-              grn_edges_delete(ctx, edge);
-            }
-            // todo : log stat
-          }
-          for (;;) {
-            MUTEX_LOCK(q_mutex);
-            if (nthreads == nfthreads) { break; }
-            MUTEX_UNLOCK(q_mutex);
-            usleep(1000);
-          }
-          {
-            grn_edge *edge;
-            GRN_HASH_EACH(ctx, grn_edges, id, NULL, NULL, &edge, {
-                grn_obj *obj;
-              while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->send_old))) {
-                grn_msg_close(&edge->ctx, obj);
-              }
-              while ((obj = (grn_obj *)grn_com_queue_deque(ctx, &edge->recv_new))) {
-                grn_msg_close(ctx, obj);
-              }
-              grn_ctx_fin(&edge->ctx);
-              if (edge->com->has_sid) {
-                grn_com_close(ctx, edge->com);
-              }
-              grn_edges_delete(ctx, edge);
-            });
-          }
-          {
-            grn_com *com;
-            GRN_HASH_EACH(ctx, ev.hash, id, NULL, NULL, &com, { grn_com_close(ctx, com); });
-          }
-          rc = 0;
-        } else {
-          fprintf(stderr, "grn_com_sopen failed (%s:%d): %s\n",
-                  bind_address, port, ctx->errbuf);
-        }
-        grn_edges_fin(ctx);
-      }
+      exit_code = run_server(ctx, db, &ev, g_dispatcher, g_handler);
       grn_obj_close(ctx, db);
     } else {
       fprintf(stderr, "db open failed (%s)\n", path);
@@ -1922,64 +1968,7 @@ g_server(char *path)
     fprintf(stderr, "grn_com_event_init failed\n");
   }
   grn_ctx_fin(ctx);
-  return rc;
-}
-
-static int
-do_daemon(char *path)
-{
-  int rc;
-#ifndef WIN32
-  pid_t pid;
-  FILE *pidfile = NULL;
-
-  switch (fork()) {
-  case 0:
-    break;
-  case -1:
-    perror("fork");
-    return EXIT_FAILURE;
-  default:
-    wait(NULL);
-    return EXIT_SUCCESS;
-  }
-  if (pidfile_path) {
-    pidfile = fopen(pidfile_path, "w");
-  }
-  switch ((pid = fork())) {
-  case 0:
-    break;
-  case -1:
-    perror("fork");
-    return EXIT_FAILURE;
-  default:
-    if (!pidfile) {
-      fprintf(stderr, "%d\n", pid);
-    } else {
-      fprintf(pidfile, "%d\n", pid);
-      fclose(pidfile);
-    }
-    _exit(EXIT_SUCCESS);
-  }
-  {
-    int null_fd = GRN_OPEN("/dev/null", O_RDWR, 0);
-    if (null_fd != -1) {
-      dup2(null_fd, 0);
-      dup2(null_fd, 1);
-      dup2(null_fd, 2);
-      if (null_fd > 2) { GRN_CLOSE(null_fd); }
-    }
-  }
-#endif /* WIN32 */
-  rc = do_server(path);
-#ifndef WIN32
-  if (pidfile) {
-    fclose(pidfile);
-    unlink(pidfile_path);
-  }
-#endif
-
-  return rc;
+  return exit_code;
 }
 
 enum {
@@ -2367,7 +2356,7 @@ show_version(void)
 #ifdef WITH_MECAB
   printf(",mecab");
 #endif
-#ifdef HAVE_MESSAGE_PACK
+#ifdef WITH_MESSAGE_PACK
   printf(",msgpack");
 #endif
 #ifndef NO_ZLIB
@@ -2480,7 +2469,8 @@ main(int argc, char **argv)
     *default_command_version_arg = NULL,
     *default_match_escalation_threshold_arg = NULL;
   const char *config_path = NULL;
-  int r, i, mode = mode_alone;
+  int exit_code = EXIT_SUCCESS;
+  int i, mode = mode_alone;
   static grn_str_getopt_opt opts[] = {
     {'p', "port", NULL, 0, getopt_op_none},
     {'e', "encoding", NULL, 0, getopt_op_none},
@@ -2516,7 +2506,7 @@ main(int argc, char **argv)
   opts[11].arg = &protocol_arg;
   opts[13].arg = &log_path_arg;
   opts[14].arg = &query_log_path_arg;
-  opts[15].arg = &pidfile_path;
+  opts[15].arg = &pid_file_path;
   opts[16].arg = &config_path;
   opts[18].arg = &cache_limit_arg;
   opts[19].arg = &input_path;
@@ -2786,7 +2776,7 @@ main(int argc, char **argv)
     cache_limit = default_cache_limit;
   }
 
-#ifdef HAVE_LIBEDIT
+#ifdef WITH_LIBEDIT
   if (!batchmode) {
     line_editor_init(argc, argv);
   }
@@ -2824,27 +2814,27 @@ main(int argc, char **argv)
   useql = (mode & MODE_USE_QL);
   switch (mode & MODE_MASK) {
   case mode_alone :
-    r = do_alone(argc - i, argv + i);
+    exit_code = do_alone(argc - i, argv + i);
     break;
   case mode_client :
-    r = do_client(argc - i, argv + i);
+    exit_code = do_client(argc - i, argv + i);
     break;
   case mode_daemon :
-    r = do_daemon(argc > i ? argv[i] : NULL);
-    break;
+    is_daemon_mode = GRN_TRUE;
+    /* fallthru */
   case mode_server :
-    r = do_server(argc > i ? argv[i] : NULL);
+    exit_code = do_server(argc > i ? argv[i] : NULL);
     break;
   default:
-    r = -1;
+    exit_code = EXIT_FAILURE;
     break;
   }
 
-#ifdef HAVE_LIBEDIT
+#ifdef WITH_LIBEDIT
   if (!batchmode) {
     line_editor_fin();
   }
 #endif
   grn_fin();
-  return r;
+  return exit_code;
 }
